@@ -402,10 +402,52 @@ void GEdge::writeGEO(FILE *fp)
   if(meshAttributes.reverseMesh) fprintf(fp, "Reverse Line {%d};\n", tag());
 }
 
+void GEdge::writePY(FILE *fp)
+{
+  // This is by no means complete - merely a placeholder for a future
+  // implementation
+
+  if(!getBeginVertex() || !getEndVertex() || geomType() == DiscreteCurve ||
+     geomType() == BoundaryLayerCurve)
+    return;
+
+  const char *factory = getNativeType() == OpenCascadeModel ? "occ" : "geo";
+
+  if(geomType() == Line) {
+    fprintf(fp, "gmsh.model.%s.addLine(%d, %d, %d)\n", factory,
+            getBeginVertex()->tag(), getEndVertex()->tag(), tag());
+  }
+  else {
+    // TODO
+  }
+}
+
+bool GEdge::storeSTLAsMesh()
+{
+  // as the STL might be non-conforming, we make no effort to have a conformal
+  // mesh - nodes will not be classified on boundaries, and not shared with
+  // adjacent entities
+  if(stl_vertices_xyz.size()) {
+    for(std::size_t i = 0; i < stl_vertices_xyz.size(); i++) {
+      SPoint3 &p(stl_vertices_xyz[i]);
+      mesh_vertices.push_back(new MVertex(p.x(), p.y(), p.z(), this));
+    }
+  }
+  else {
+    return false;
+  }
+  for(std::size_t i = 0; i < mesh_vertices.size() - 1; i++) {
+    lines.push_back(new MLine(mesh_vertices[i], mesh_vertices[i + 1]));
+  }
+  return true;
+}
+
 bool GEdge::containsPoint(const SPoint3 &pt) const
 {
   if(geomType() == BoundaryLayerCurve) return false;
-  return containsParam(parFromPoint(pt));
+  double t;
+  if(!XYZToU(pt.x(), pt.y(), pt.z(), t, 1, false)) return false;
+  return containsParam(t);
 }
 
 bool GEdge::containsParam(double pt) const
@@ -660,6 +702,15 @@ std::list<GRegion *> GEdge::regions() const
   return ret;
 }
 
+bool GEdge::isOrphan()
+{
+  if(model()->getNumRegions())
+    return regions().empty();
+  else if(model()->getNumFaces())
+    return faces().empty();
+  return false;
+}
+
 void GEdge::relocateMeshVertices()
 {
   for(std::size_t i = 0; i < mesh_vertices.size(); i++) {
@@ -725,7 +776,8 @@ void GEdge::addElement(int type, MElement *e)
 {
   switch(type) {
   case TYPE_LIN: addLine(reinterpret_cast<MLine *>(e)); break;
-  default: Msg::Error("Trying to add unsupported element in curve %d", tag());
+  default:
+    Msg::Error("Trying to add unsupported element in curve %d", tag());
   }
 }
 
@@ -742,6 +794,15 @@ void GEdge::removeElement(int type, MElement *e)
   }
 }
 
+void GEdge::removeElements(int type)
+{
+  switch(type) {
+  case TYPE_LIN: lines.clear(); break;
+  default:
+    Msg::Error("Trying to remove unsupported elements in curve %d", tag());
+  }
+}
+
 void GEdge::discretize(double tol, std::vector<SPoint3> &dpts,
                        std::vector<double> &ts)
 {
