@@ -121,7 +121,7 @@ void classifyFaces(GModel *gm, double curveAngleThreshold)
   }
   if(ho) {
     Msg::Warning("Reverting to first order mesh for classification");
-    gm->setOrderN(1, false, false);
+    gm->setOrderN(1, 0, 0, 0);
   }
 
   // create a structure from mesh edges to geometrical curves, and remove curves
@@ -732,9 +732,7 @@ static int isTriangulationParametrizable(const std::vector<MTriangle *> &t,
 
   std::set<MVertex *> v;
   std::map<MEdge, int, MEdgeLessThan> e;
-  double surf = 0;
   for(std::size_t i = 0; i < t.size(); ++i) {
-    surf += t[i]->getVolume();
     for(int j = 0; j < 3; j++) {
       v.insert(t[i]->getVertex(j));
       auto it = e.find(t[i]->getEdge(j));
