@@ -5,6 +5,7 @@
 //
 // Contributed by Ismail Badia.
 
+#include <stdexcept>
 #include <algorithm>
 #include "HierarchicalBasisHcurlBrick.h"
 
@@ -48,7 +49,7 @@ double HierarchicalBasisHcurlBrick::_affineCoordinate(const int &j,
   case(4): return 0.5 * (1 - v);
   case(5): return 0.5 * (1 + w);
   case(6): return 0.5 * (1 - w);
-  default: throw std::string("j must be : 1<=j<=6");
+  default: throw std::runtime_error("j must be : 1<=j<=6");
   }
 }
 
@@ -598,7 +599,7 @@ void HierarchicalBasisHcurlBrick::orientOneFace(
         }
       }
       else {
-        throw std::string("unknown typeFunction");
+        throw std::runtime_error("unknown typeFunction");
       }
     }
   }
